@@ -38,7 +38,12 @@ moduleName = "Language.Haskell.Meta.Syntax.Translate"
 -- When to use each of these isn't always clear: prefer 'todo' if unsure.
 noTH :: Show e => String -> e -> a
 noTH fun thing = error . concat $ [moduleName, ".", fun,
-  ": no TH representation for: ", show thing]
+  ": template-haskell has no representation for: ", show thing]
+
+noTHyet :: Show e => String -> String -> e -> a
+noTHyet fun minVersion thing = error . concat $ [moduleName, ".", fun,
+  ": template-haskell-", VERSION_template_haskell, " (< ", minVersion, ")",
+  " has no representation for: ", show thing]
 
 todo :: Show e => String -> e -> a
 todo fun thing = error . concat $ [moduleName, ".", fun,
@@ -149,17 +154,13 @@ instance ToLit Hs.Literal where
   toLit (Hs.PrimString a) = StringPrimL (map toWord8 a)
    where
     toWord8 = fromIntegral . ord
-#elif MIN_VERSION_template_haskell(2,5,0)
-  toLit (Hs.PrimString a) = StringPrimL a
 #else
-  toLit l@Hs.PrimString{} = noTH "toLit" l
+  toLit (Hs.PrimString a) = StringPrimL a
 #endif
   toLit (Hs.PrimInt a) = IntPrimL a
   toLit (Hs.PrimFloat a) = FloatPrimL a
   toLit (Hs.PrimDouble a) = DoublePrimL a
-#if MIN_VERSION_template_haskell(2,4,0)
   toLit (Hs.PrimWord a) = WordPrimL a
-#endif
 
 
 -----------------------------------------------------------------------------
@@ -186,7 +187,12 @@ instance ToPat Hs.Pat where
   toPat (Hs.PInfixApp p n q)= InfixP (toPat p) (toName n) (toPat q)
 #endif
   toPat (Hs.PApp n ps) = ConP (toName n) (fmap toPat ps)
-  toPat (Hs.PTuple ps) = TupP (fmap toPat ps)
+  toPat (Hs.PTuple Hs.Boxed ps) = TupP (fmap toPat ps)
+#if MIN_VERSION_template_haskell(2,6,0)
+  toPat (Hs.PTuple Hs.Unboxed ps) = UnboxedTupP (fmap toPat ps)
+#else
+  toPat p@(Hs.PTuple Hs.Unboxed _) = noTHyet "toPat" "2.6.0" p
+#endif
   toPat (Hs.PList ps) = ListP (fmap toPat ps)
 #if MIN_VERSION_template_haskell(2,7,0)  
   toPat (Hs.PParen p) = ParensP (toPat p)  
@@ -199,9 +205,7 @@ instance ToPat Hs.Pat where
   toPat (Hs.PWildCard) = WildP
   toPat (Hs.PIrrPat p) = TildeP (toPat p)
   toPat (Hs.PatTypeSig _ p t) = SigP (toPat p) (toType t)
-#if MIN_VERSION_template_haskell(2,5,0)
   toPat (Hs.PViewPat e p) = ViewP (toExp e) (toPat p)
-#endif
   -- regular pattern
   toPat p@Hs.PRPat{} = noTH "toPat" p
   -- XML stuff
@@ -209,9 +213,7 @@ instance ToPat Hs.Pat where
   toPat p@Hs.PXETag{} = noTH "toPat" p
   toPat p@Hs.PXPcdata{} = noTH "toPat" p
   toPat p@Hs.PXPatTag{} = noTH "toPat" p
-#if MIN_VERSION_template_haskell(2,4,0)
   toPat (Hs.PBangPat p) = BangP (toPat p)
-#endif /* MIN_VERSION_template_haskell(2,4,0) */
   toPat p = todo "toPat" p
 
 -----------------------------------------------------------------------------
@@ -246,7 +248,12 @@ instance ToExp Hs.Exp where
   toExp (Hs.If a b c)              = CondE (toExp a) (toExp b) (toExp c)
   toExp (Hs.Do ss)                 = DoE (map toStmt ss)
   toExp e@(Hs.MDo _)               = noTH "toExp" e
-  toExp (Hs.Tuple xs)              = TupE (fmap toExp xs)
+  toExp (Hs.Tuple Hs.Boxed xs)     = TupE (fmap toExp xs)
+#if MIN_VERSION_template_haskell(2,6,0)
+  toExp (Hs.Tuple Hs.Unboxed xs)   = UnboxedTupE (fmap toExp xs)
+#else
+  toExp e@(Hs.Tuple Hs.Unboxed _)  = noTHyet "toExp" "2.6.0" e
+#endif
   toExp (Hs.List xs)               = ListE (fmap toExp xs)
 #if MIN_VERSION_template_haskell(2,7,0)
   toExp (Hs.Paren e)               = ParensE (toExp e)
@@ -302,13 +309,10 @@ instance ToName Hs.TyVarBind where
 instance ToName Name where
   toName = id
 
-#if MIN_VERSION_template_haskell(2,4,0)
 instance ToName TyVarBndr where
   toName (PlainTV n) = n
   toName (KindedTV n _) = n
-#endif /* !MIN_VERSION_template_haskell(2,4,0) */
 
-#if MIN_VERSION_template_haskell(2,4,0)
 #if MIN_VERSION_template_haskell(2,8,0)
 
 instance ToType Hs.Kind where
@@ -328,20 +332,13 @@ toKind Hs.KindStar = StarK
 toKind (Hs.KindFn k1 k2) = ArrowK (toKind k1) (toKind k2)
 toKind (Hs.KindParen kp) = toKind kp
 toKind k@Hs.KindBang = noTH "toKind" k
-toKind k@Hs.KindVar{} = noTH "toKind" k
+toKind k@Hs.KindVar{} = noTHyet "toKind" "2.8.0" k
 
 #endif /* !MIN_VERSION_template_haskell(2,8,0) */
-#endif /* !MIN_VERSION_template_haskell(2,4,0) */
 
-#if MIN_VERSION_template_haskell(2,4,0)
 toTyVar :: Hs.TyVarBind -> TyVarBndr
 toTyVar (Hs.KindedVar n k) = KindedTV (toName n) (toKind k)
 toTyVar (Hs.UnkindedVar n) = PlainTV (toName n)
-#else /* !MIN_VERSION_template_haskell(2,4,0) */
-toTyVar :: Hs.TyVarBind -> Name
-toTyVar (Hs.KindedVar n _) = toName n
-toTyVar (Hs.UnkindedVar n) = toName n
-#endif /* !MIN_VERSION_template_haskell(2,4,0) */
 
 instance ToType Hs.Type where
   toType (Hs.TyForall tvbM cxt t) = ForallT (maybe [] (fmap toTyVar) tvbM) (toCxt cxt) (toType t)
@@ -354,7 +351,7 @@ instance ToType Hs.Type where
 #if MIN_VERSION_template_haskell(2,6,0)
       Hs.Unboxed -> UnboxedTupleT
 #else
-      Hs.Unboxed -> noTH "toType TyTuple" (Hs.TyTuple b ts)
+      Hs.Unboxed -> noTHyet "toType TyTuple" "2.6.0" (Hs.TyTuple b ts)
 #endif
   toType (Hs.TyApp a b) = AppT (toType a) (toType b)
   toType (Hs.TyVar n) = VarT (toName n)
@@ -373,17 +370,10 @@ a .->. b = AppT (AppT ArrowT a) b
 toCxt :: Hs.Context -> Cxt
 toCxt = fmap toPred
  where
-#if MIN_VERSION_template_haskell(2,4,0)
   toPred (Hs.ClassA n ts) = ClassP (toName n) (fmap toType ts)
   toPred (Hs.InfixA t1 n t2) = ClassP (toName n) (fmap toType [t1, t2])
   toPred (Hs.EqualP t1 t2) = EqualP (toType t1) (toType t2)
   toPred a@Hs.IParam{} = noTH "toCxt" a
-#else
-  toPred (Hs.ClassA n ts) = foldAppT (ConT (toName n)) (fmap toType ts)
-  toPred (Hs.InfixA t1 n t2) = foldAppT (ConT (toName n)) (fmap toType [t1, t2])
-  toPred a@Hs.EqualP{} = noTH "toCxt" a
-  toPred a@Hs.IParam{} = noTH "toCxt" a
-#endif
 
 foldAppT :: Type -> [Type] -> Type
 foldAppT t ts = foldl' AppT t ts
@@ -440,7 +430,6 @@ instance ToDec Hs.Decl where
     = let xs = fmap (flip SigD (toType t) . toName) ns
       in case xs of x:_ -> x; [] -> error "toDec: malformed TypeSig!"
 
-#if MIN_VERSION_template_haskell(2,4,0)
 #if MIN_VERSION_template_haskell(2,8,0)
 
   toDec (Hs.InlineConlikeSig _ act qn) = PragmaD $
@@ -466,8 +455,6 @@ instance ToDec Hs.Decl where
   toDec (Hs.DataFamDecl _ _ n ns k)
     = FamilyD DataFam (toName n) (fmap toTyVar ns) (fmap toKind k)
 
-#endif /* MIN_VERSION_template_haskell(2,4,0) */
-
   toDec a@(Hs.FunBind mtchs)                           = hsMatchesToFunD mtchs
   toDec (Hs.PatBind _ p tM rhs bnds)                   = ValD ((maybe id
                                                                       (flip SigP . toType)
@@ -597,11 +584,7 @@ instance ToDecs Hs.Decl where
   toDecs a = [toDec a]
 
 collectVars e = case e of
-#if MIN_VERSION_template_haskell(2,4,0)
   VarT n -> [PlainTV n]
-#else /* !MIN_VERSION_template_haskell(2,4,0) */
-  VarT n -> [n]
-#endif /* !MIN_VERSION_template_haskell(2,4,0) */
   AppT t1 t2 -> nub $ collectVars t1 ++ collectVars t2
   ForallT ns _ t -> collectVars t \\ ns
   _          -> []
